@@ -58,7 +58,7 @@ struct mysql_database_s /* {{{ */
 	_Bool slave_sql_running;
 
 	MYSQL *con;
-	int    state;
+	_Bool  is_connected;
 };
 typedef struct mysql_database_s mysql_database_t; /* }}} */
 
@@ -192,9 +192,9 @@ static int mysql_config_database (oconfig_item_t *ci) /* {{{ */
 		ud.data = (void *) db;
 		ud.free_func = mysql_database_free;
 
-		if (db->database != NULL)
+		if (db->instance != NULL)
 			ssnprintf (cb_name, sizeof (cb_name), "mysql-%s",
-					db->database);
+					db->instance);
 		else
 			sstrncpy (cb_name, "mysql", sizeof (cb_name));
 
@@ -237,30 +237,28 @@ static int mysql_config (oconfig_item_t *ci) /* {{{ */
 
 static MYSQL *getconnection (mysql_database_t *db)
 {
-	if (db->state != 0)
+	if (db->is_connected)
 	{
-		int err;
-		if ((err = mysql_ping (db->con)) != 0)
-		{
-			/* Assured by "mysql_config_database" */
-			assert (db->instance != NULL);
-			WARNING ("mysql_ping failed for instance \"%s\": %s",
-					db->instance,
-					mysql_error (db->con));
-			db->state = 0;
-		}
-		else
-		{
-			db->state = 1;
-			return (db->con);
-		}
-	}
+		int status;
 
-	if ((db->con = mysql_init (db->con)) == NULL)
+		status = mysql_ping (db->con);
+		if (status == 0)
+			return (db->con);
+
+		WARNING ("mysql plugin: Lost connection to instance \"%s\": %s",
+				db->instance, mysql_error (db->con));
+	}
+	db->is_connected = 0;
+
+	if (db->con == NULL)
 	{
-		ERROR ("mysql_init failed: %s", mysql_error (db->con));
-		db->state = 0;
-		return (NULL);
+		db->con = mysql_init (NULL);
+		if (db->con == NULL)
+		{
+			ERROR ("mysql plugin: mysql_init failed: %s",
+					mysql_error (db->con));
+			return (NULL);
+		}
 	}
 
 	if (mysql_real_connect (db->con, db->host, db->user, db->pass,
@@ -271,20 +269,18 @@ static MYSQL *getconnection (mysql_database_t *db)
 				(db->database != NULL) ? db->database : "<none>",
 				(db->host != NULL) ? db->host : "localhost",
 				mysql_error (db->con));
-		db->state = 0;
 		return (NULL);
 	}
-	else
-	{
-		INFO ("mysql plugin: Successfully connected to database %s "
-				"at server %s (server version: %s, protocol version: %d)",
-				(db->database != NULL) ? db->database : "<none>",
-				mysql_get_host_info (db->con),
-				mysql_get_server_info (db->con),
-				mysql_get_proto_info (db->con));
-		db->state = 1;
-		return (db->con);
-	}
+
+	INFO ("mysql plugin: Successfully connected to database %s "
+			"at server %s (server version: %s, protocol version: %d)",
+			(db->database != NULL) ? db->database : "<none>",
+			mysql_get_host_info (db->con),
+			mysql_get_server_info (db->con),
+			mysql_get_proto_info (db->con));
+
+	db->is_connected = 1;
+	return (db->con);
 } /* static MYSQL *getconnection (mysql_database_t *db) */
 
 static void set_host (mysql_database_t *db, char *buf, size_t buflen)
@@ -403,6 +399,7 @@ static int mysql_read_master_stats (mysql_database_t *db, MYSQL *con)
 	{
 		ERROR ("mysql plugin: Failed to get master statistics: "
 				"`%s' did not return any rows.", query);
+		mysql_free_result (res);
 		return (-1);
 	}
 
@@ -411,6 +408,7 @@ static int mysql_read_master_stats (mysql_database_t *db, MYSQL *con)
 	{
 		ERROR ("mysql plugin: Failed to get master statistics: "
 				"`%s' returned less than two columns.", query);
+		mysql_free_result (res);
 		return (-1);
 	}
 
@@ -454,6 +452,7 @@ static int mysql_read_slave_stats (mysql_database_t *db, MYSQL *con)
 	{
 		ERROR ("mysql plugin: Failed to get slave statistics: "
 				"`%s' did not return any rows.", query);
+		mysql_free_result (res);
 		return (-1);
 	}
 
@@ -462,6 +461,7 @@ static int mysql_read_slave_stats (mysql_database_t *db, MYSQL *con)
 	{
 		ERROR ("mysql plugin: Failed to get slave statistics: "
 				"`%s' returned less than 33 columns.", query);
+		mysql_free_result (res);
 		return (-1);
 	}
 
@@ -534,7 +534,7 @@ static int mysql_read_slave_stats (mysql_database_t *db, MYSQL *con)
 			ssnprintf (n.message, sizeof (n.message),
 					"slave SQL thread started");
 			plugin_dispatch_notification (&n);
-			db->slave_sql_running = 0;
+			db->slave_sql_running = 1;
 		}
 	}
 
