@@ -250,7 +250,7 @@ int create_putval (char *ret, size_t ret_len, /* {{{ */
 			buffer_ident,
 			(vl->interval > 0)
 			? CDTIME_T_TO_DOUBLE (vl->interval)
-			: CDTIME_T_TO_DOUBLE (interval_g),
+			: CDTIME_T_TO_DOUBLE (plugin_get_interval ()),
 			buffer_values);
 
 	return (0);
